@@ -8,7 +8,7 @@
 #
 #  KeepNote
 #  Copyright (c) 2008-2011 Matt Rasmussen
-#  Author: Matt Rasmussen <rasmus@mit.edu>
+#  Author: Matt Rasmussen <rasmus@alum.mit.edu>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -95,7 +95,7 @@ except ImportError:
 PROGRAM_NAME = u"KeepNote"
 PROGRAM_VERSION_MAJOR = 0
 PROGRAM_VERSION_MINOR = 7
-PROGRAM_VERSION_RELEASE = 6
+PROGRAM_VERSION_RELEASE = 7
 PROGRAM_VERSION = (PROGRAM_VERSION_MAJOR,
                    PROGRAM_VERSION_MINOR,
                    PROGRAM_VERSION_RELEASE)
@@ -114,6 +114,7 @@ COPYRIGHT = u"Copyright Matt Rasmussen 2011."
 TRANSLATOR_CREDITS = (
     u"Chinese: hu dachuan <hdccn@sina.com>\n"
     u"French: tb <thibaut.bethune@gmail.com>\n"
+    u"French: Sebastien KALT <skalt@throka.org>\n"
     u"German: Jan Rimmek <jan.rimmek@mhinac.de>\n"
     u"Japanese: Toshiharu Kudoh <toshi.kd2@gmail.com>\n"
     u"Italian: Davide Melan <davide.melan@gmail.com>\n"
@@ -129,8 +130,6 @@ TRANSLATOR_CREDITS = (
 
 
 BASEDIR = os.path.dirname(unicode(__file__, FS_ENCODING))
-IMAGE_DIR = u"images"
-NODE_ICON_DIR = os.path.join(IMAGE_DIR, u"node_icons")
 PLATFORM = None
 
 USER_PREF_DIR = u"keepnote"
@@ -142,11 +141,6 @@ USER_EXTENSIONS_DATA_DIR = u"extensions_data"
 PORTABLE_FILE = u"portable.txt"
 
 
-DEFAULT_WINDOW_SIZE = (1024, 600)
-DEFAULT_WINDOW_POS = (-1, -1)
-DEFAULT_AUTOSAVE_TIME = 10 * 1000 # 10 sec (in msec)
-
-
 
 #=============================================================================
 # application resources
@@ -749,6 +743,8 @@ class KeepNote (object):
         self._extensions = {}
         self._disabled_extensions = []
 
+        # listeners
+        self._listeners = {}
 
 
     def init(self):
@@ -795,7 +791,6 @@ class KeepNote (object):
         self.pref.get("extensions", define=True)
 
 
-
     def save_preferences(self):
         """Save information into preferences"""
         
@@ -880,7 +875,10 @@ class KeepNote (object):
     def close_all_notebook(self, notebook, save=True):
         """Close all instances of a notebook"""
 
-        notebook.close(save)
+        try:
+            notebook.close(save)
+        except:
+            keepnote.log_error()
 
         notebook.closing_event.remove(self._on_closing_notebook)
         del self._notebook_count[notebook]
@@ -979,8 +977,18 @@ class KeepNote (object):
 
 
     #================================
-    # external apps
+    # listeners
 
+    def get_listeners(self, key):
+        listeners = self._listeners.get(key, None)
+        if listeners is None:
+            listeners = Listeners()
+            self._listeners[key] = listeners
+        return listeners
+        
+
+    #================================
+    # external apps
 
     def _load_external_app_preferences(self):
         
