@@ -7,8 +7,8 @@
 
 #
 #  KeepNote
-#  Copyright (c) 2008-2009 Matt Rasmussen
-#  Author: Matt Rasmussen <rasmus@mit.edu>
+#  Copyright (c) 2008-2011 Matt Rasmussen
+#  Author: Matt Rasmussen <rasmus@alum.mit.edu>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -65,6 +65,7 @@ from keepnote.gui import \
      ToggleAction, \
      add_actions, \
      CONTEXT_MENU_ACCEL_PATH, \
+     CLIPBOARD_NAME, \
      FileChooserDialog, \
      init_key_shortcuts, \
      UIManager
@@ -84,8 +85,14 @@ from keepnote.gui.tabbed_viewer import TabbedViewer
 
 _ = keepnote.translate
 
-CLIPBOARD_NAME = "CLIPBOARD"
+#=============================================================================
+# constants
 
+DEFAULT_WINDOW_SIZE = (1024, 600)
+DEFAULT_WINDOW_POS = (-1, -1)
+
+
+#=============================================================================
 
 
 class KeepNoteWindow (gtk.Window):
@@ -124,7 +131,7 @@ class KeepNoteWindow (gtk.Window):
     def init_layout(self):
         # init main window
         self.set_title(keepnote.PROGRAM_NAME)
-        self.set_default_size(*keepnote.DEFAULT_WINDOW_SIZE)
+        self.set_default_size(*DEFAULT_WINDOW_SIZE)
         self.set_icon_list(get_resource_pixbuf("keepnote-16x16.png"),
                            get_resource_pixbuf("keepnote-32x32.png"),
                            get_resource_pixbuf("keepnote-64x64.png"))
@@ -282,9 +289,9 @@ class KeepNoteWindow (gtk.Window):
         return self.viewer.get_notebook()
 
     
-    def get_current_page(self):
-        """Returns the currently selected page"""
-        return self.viewer.get_current_page()
+    def get_current_node(self):
+        """Returns the currently selected node"""
+        return self.viewer.get_current_node()
         
         
 
@@ -397,7 +404,7 @@ class KeepNoteWindow (gtk.Window):
 
         # notebook
         window_size = p.get("window", "window_size", 
-                            default=keepnote.DEFAULT_WINDOW_SIZE)
+                            default=DEFAULT_WINDOW_SIZE)
         window_maximized = p.get("window", "window_maximized", default=True)
 
 
@@ -532,7 +539,7 @@ class KeepNoteWindow (gtk.Window):
                 filechooser.response(gtk.RESPONSE_OK)
                         
         dialog.connect("current-folder-changed", on_folder_changed)
-
+        
         path = self._app.get_default_path("new_notebook_path")
         if os.path.exists(path):
             dialog.set_current_folder(path)
@@ -546,21 +553,64 @@ class KeepNoteWindow (gtk.Window):
         file_filter.add_pattern("*")
         file_filter.set_name(_("All files (*.*)"))
         dialog.add_filter(file_filter)
-        
+
         response = dialog.run()
         
         if response == gtk.RESPONSE_OK:
-            path = dialog.get_current_folder()
+
+            path = ensure_unicode(dialog.get_current_folder(), FS_ENCODING)
             if path:
                 self._app.pref.set("default_paths", "new_notebook_path", 
                                    os.path.dirname(path))
 
-            notebook_file = unicode_gtk(dialog.get_filename())
+            notebook_file = ensure_unicode(dialog.get_filename(), FS_ENCODING)
             if notebook_file:
                 self.open_notebook(notebook_file)
 
         dialog.destroy()
 
+
+    
+    def on_open_notebook_url(self):
+        """Launches Open NoteBook from URL dialog"""
+        
+        dialog = gtk.Dialog("Open Notebook from URL", self,
+                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+        
+        p = dialog.get_content_area()
+        
+        h = gtk.HBox()
+        h.show()
+        p.pack_start(h, expand=False, fill=True, padding=0)
+
+        # url label
+        l = gtk.Label("URL: ")
+        l.show()
+        h.pack_start(l, expand=False, fill=True, padding=0)
+
+        # url entry
+        entry = gtk.Entry()
+        entry.set_width_chars(80)
+        entry.connect("activate", lambda w: 
+                      dialog.response(gtk.RESPONSE_OK))
+        entry.show()
+        h.pack_start(entry, expand=True, fill=True, padding=0)
+        
+
+        # actions
+        dialog.add_button("_Cancel", gtk.RESPONSE_CANCEL)
+        dialog.add_button("_Open", gtk.RESPONSE_OK)
+
+        response = dialog.run()
+        
+        if response == gtk.RESPONSE_OK:
+            url = unicode_gtk(entry.get_text())
+            if url:
+                self.open_notebook(url)
+
+        dialog.destroy()
+
+
     
     def _on_close(self):
         """Callback for window close"""
@@ -889,7 +939,7 @@ class KeepNoteWindow (gtk.Window):
         else:
             title = notebook.get_attr("title", u"")
             if node is None:
-                node = self.get_current_page()
+                node = self.get_current_node()
             if node is not None:
                 title += u": " + node.get_attr("title", "")
 
@@ -1078,7 +1128,7 @@ class KeepNoteWindow (gtk.Window):
         if os.path.exists(license_file):
             about.set_license(open(license_file).read())
 
-        #about.set_authors(["Matt Rasmussen <rasmus@mit.edu>"])
+        #about.set_authors(["Matt Rasmussen <rasmus@alum.mit.edu>"])
         
 
         about.set_transient_for(self)
@@ -1155,6 +1205,10 @@ class KeepNoteWindow (gtk.Window):
              "", _("Close the current notebook"),
              lambda w: self._app.close_all_notebook(self.get_notebook())),
             
+            ("Empty Trash", gtk.STOCK_DELETE, _("Empty _Trash"),
+             "", None,
+             lambda w: self.on_empty_trash()),
+
             ("Export", None, _("_Export Notebook")),
 
             ("Import", None, _("_Import Notebook")),
@@ -1189,10 +1243,11 @@ class KeepNoteWindow (gtk.Window):
             ("Paste", gtk.STOCK_PASTE, None,
              "<control>V", None,
              lambda w: self.on_paste()),
-
-            ("Empty Trash", gtk.STOCK_DELETE, _("Empty _Trash"),
+            
+            ("KeepNote Preferences", gtk.STOCK_PREFERENCES, _("_Preferences"),
              "", None,
-             lambda w: self.on_empty_trash()),
+             lambda w: self._app.app_options_dialog.show(self)),
+
             
             #========================================
             ("Search", None, _("_Search")),
@@ -1207,6 +1262,8 @@ class KeepNoteWindow (gtk.Window):
             #========================================
             ("View", None, _("_View")),
 
+            ("View Note As", gtk.STOCK_OPEN, _("_View Note As")),
+
             ("View Note in File Explorer", gtk.STOCK_OPEN,
              _("View Note in File Explorer"),
              "", None,
@@ -1238,10 +1295,10 @@ class KeepNoteWindow (gtk.Window):
             ("Update Notebook Index", None, _("_Update Notebook Index"),
              "", None,
              lambda w: self.update_index(clear=True)),
-            
-            ("KeepNote Preferences", gtk.STOCK_PREFERENCES, _("_Preferences"),
+
+            ("Open Notebook URL", None, _("_Open Notebook from URL"),
              "", None,
-             lambda w: self._app.app_options_dialog.show(self)),
+             lambda w: self.on_open_notebook_url()),
 
             #=========================================
             ("Window", None, _("Window")),
@@ -1347,6 +1404,7 @@ class KeepNoteWindow (gtk.Window):
   <menu action="Tools">
     <placeholder name="Viewer"/>
     <menuitem action="Update Notebook Index"/>
+    <menuitem action="Open Notebook URL"/>
     <placeholder name="Extensions"/>
   </menu>
 
@@ -1592,7 +1650,8 @@ class SearchBox (gtk.Entry):
             try:
                 nodes = (notebook.get_node_by_id(nodeid)
                          for nodeid in 
-                         notebook.search_node_contents(" ".join(words)))
+                         notebook.search_node_contents(" ".join(words))
+                         if nodeid)
             except:
                 keepnote.log_error()
 
