@@ -7,8 +7,8 @@
 
 #
 #  KeepNote
-#  Copyright (c) 2008-2009 Matt Rasmussen
-#  Author: Matt Rasmussen <rasmus@mit.edu>
+#  Copyright (c) 2008-2011 Matt Rasmussen
+#  Author: Matt Rasmussen <rasmus@alum.mit.edu>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,151 +45,33 @@ except Exception, e:
 # keepnote imports
 import keepnote
 import keepnote.notebook
-
+from keepnote.notebook.connection.index import AttrIndex, NodeIndex
 
 
 # index filename
 INDEX_FILE = u"index.sqlite"
 INDEX_VERSION = 3
 
-
-NULL = object()
-
-
-
-def match_words(infile, words):
-    """Returns True if all of the words in list 'words' appears in the
-       node title or data file"""
-
-    matches = dict.fromkeys(words, False)
-
-    for line in infile:
-        line = line.lower()
-        for word in words:
-            if word in line:
-                matches[word] = True
-
-    # return True if all words are found (AND)
-    for val in matches.itervalues():
-        if not val:
-            return False
-    
-    return True
-
-
-def read_data_as_plain_text(conn, nodeid):
-    """Iterates over the lines of the data file as plain text"""
-    try:
-        infile = conn.open_file(
-            nodeid, keepnote.notebook.PAGE_DATA_FILE, "r", codec="utf-8")
-        for line in keepnote.notebook.read_data_as_plain_text(infile):
-            yield line
-        infile.close()
-    except:
-        pass
-
-
-
-class AttrIndex (object):
-    """Indexing information for an attribute"""
-
-    def __init__(self, name, type, multivalue=False, index_value=False):
-        self._name = name
-        self._type = type
-        self._table_name = "Attr_" + name
-        self._index_name = "IdxAttr_" + name + "_nodeid"
-        self._multivalue = multivalue
-        self._index_value = index_value
-        self._index_value_name = "IdxAttr_" + name + "_value"
-
-
-    def get_name(self):
-        return self._name
-
-    def get_table_name(self):
-        return self._table_name
-
-    def is_multivalue(self):
-        return self._multivalue
-
-    def init(self, cur):
-        """Initialize attribute index for database"""
-
-        # multivalue is not implemented yet
-        assert not self._multivalue
-
-        cur.execute(u"""CREATE TABLE IF NOT EXISTS %s
-                           (nodeid TEXT,
-                            value %s,
-                            UNIQUE(nodeid) ON CONFLICT REPLACE);
-                        """ % (self._table_name, self._type))
-        cur.execute(u"""CREATE INDEX IF NOT EXISTS %s
-                           ON %s (nodeid);""" % (self._index_name,
-                                                 self._table_name))
-
-        if self._index_value:
-            cur.execute(u"""CREATE INDEX IF NOT EXISTS %s
-                           ON %s (value);""" % (self._index_value_name,
-                                                self._table_name))
-
-    def drop(self, cur):
-        cur.execute(u"DROP TABLE IF EXISTS %s" % self._table_name)
-            
-
-    def add_node(self, cur, nodeid, attr):
-        val = attr.get(self._name, NULL)
-        if val is not NULL:
-            self.set(cur, nodeid, val)
-
-
-    def remove_node(self, cur, nodeid):
-        """Remove node from index"""
-        cur.execute(u"DELETE FROM %s WHERE nodeid=?" % self._table_name, 
-                    (nodeid,))
-
-
-    def get(self, cur, nodeid):
-        """Get information for a node from the index"""
-        cur.execute(u"""SELECT value FROM %s WHERE nodeid = ?""" % 
-                    self._table_name, (nodeid,))
-        values = [row[0] for row in cur.fetchall()]
-
-        # return value
-        if self._multivalue:
-            return values
-        else:
-            if len(values) == 0:
-                return None
-            else:
-                return values[0]
-
-    def set(self, cur, nodeid, value):
-        """Set the information for a node in the index"""
-
-        # insert new row
-        cur.execute(u"""INSERT INTO %s VALUES (?, ?)""" % self._table_name,
-                        (nodeid, value))
+#=============================================================================
 
 
 # TODO: remove uniroot
 
-class NoteBookIndex (object):
+class NoteBookIndex (NodeIndex):
     """Index for a NoteBook"""
 
     def __init__(self, conn, index_file):
-        self._nconn = conn
+        NodeIndex.__init__(self, conn)
         self._index_file = index_file
         self._uniroot = keepnote.notebook.UNIVERSAL_ROOT
-        self._attrs = {}
+        self.con = None     # sqlite connection
+        self.cur = None     # sqlite cursor
+
 
         # index state/capabilities
         self._need_index = False
         self._corrupt = False
-        self._has_fulltext = False
         
-        self.con = None # sqlite connection
-        self.cur = None # sqlite cursor
-
         # start index
         self.open()
 
@@ -302,44 +184,17 @@ class NoteBookIndex (object):
                             symlink BOOLEAN,
                             UNIQUE(nodeid) ON CONFLICT REPLACE);
                         """)
-
             con.execute(u"""CREATE INDEX IF NOT EXISTS IdxNodeGraphNodeid 
                            ON NodeGraph (nodeid);""")
             con.execute(u"""CREATE INDEX IF NOT EXISTS IdxNodeGraphParentid 
                            ON NodeGraph (parentid);""")
             
+            # init attribute indexes
+            self.init_attrs(self.cur)
+            
 
-            # full text table
-            try:
-                # test for fts3 availability
-                con.execute(u"DROP TABLE IF EXISTS fts3test;")
-                con.execute(
-                    "CREATE VIRTUAL TABLE fts3test USING fts3(col TEXT);")
-                con.execute("DROP TABLE fts3test;")
-
-                # create fulltext table if it does not already exist
-                if not list(con.execute(u"""SELECT 1 FROM sqlite_master 
-                                   WHERE name == 'fulltext';""")):
-                    con.execute(u"""CREATE VIRTUAL TABLE 
-                                fulltext USING 
-                                fts3(nodeid TEXT, content TEXT,
-                                     tokenize=porter);""")
-                self._has_fulltext = True
-            except Exception, e:
-                keepnote.log_error(e)
-                self._has_fulltext = False
-
-            # TODO: make an Attr table
-            # this will let me query whether an attribute is currently being
-            # indexed and in what table it is in.
-            #con.execute(u"""CREATE TABLE IF NOT EXISTS AttrDefs
-            #               (attr TEXT,
-            #                type );
-            #            """)
-
-            # initialize attribute tables
-            for attr in self._attrs.itervalues():
-                attr.init(self.cur)
+            # NOTE: re-claim space in the index
+            con.execute("VACUUM;")
 
             con.commit()
 
@@ -373,7 +228,10 @@ class NoteBookIndex (object):
                     
 
     def _on_corrupt(self, error, tracebk=None):
-
+        """
+        Called when database appears corrupt
+        Logs error and schedules re-indexing
+        """
         self._corrupt = True
         self._need_index = True
 
@@ -383,27 +241,12 @@ class NoteBookIndex (object):
         # TODO: reload database?
 
 
-    def add_attr(self, attr):
-        """Add indexing for a node attribute using AttrIndex"""
-        self._attrs[attr.get_name()] = attr
-        if self.cur:
-            attr.init(self.cur)
-        return attr
-
-
     def _drop_tables(self):
         """drop NodeGraph tables"""
         self.con.execute(u"DROP TABLE IF EXISTS NodeGraph")
         self.con.execute(u"DROP INDEX IF EXISTS IdxNodeGraphNodeid")
         self.con.execute(u"DROP INDEX IF EXISTS IdxNodeGraphParentid")
-        self.con.execute(u"DROP TABLE IF EXISTS fulltext;")
-        
-        # drop attribute tables
-        table_names = [x for (x,) in self.con.execute(
-            u"""SELECT name FROM sqlite_master WHERE name LIKE 'Attr_%'""")]
-
-        for table_name in table_names:
-            self.con.execute(u"""DROP TABLE %s;""" % table_name)
+        self.drop_attrs(self.cur)
         
 
     
@@ -413,11 +256,10 @@ class NoteBookIndex (object):
 
 
     def set_index_needed(self, val=True):
+        """Returns True if re-indexing is needed"""
         self._need_index = val
     
-    def has_fulltext_search(self):
-        return self._has_fulltext
-    
+
 
     #-------------------------------------
     # add/remove nodes from index
@@ -449,6 +291,7 @@ class NoteBookIndex (object):
 
 
         # perform indexing
+        # simply by walking through the tree, all nodes will index themselves
         for nodeid in preorder(conn, rootid):
             yield nodeid
 
@@ -457,6 +300,7 @@ class NoteBookIndex (object):
 
 
     def get_node_mtime(self, nodeid):
+        """Get the last indexed mtime for a node"""
         
         self.cur.execute(u"""SELECT mtime FROM NodeGraph
                              WHERE nodeid=?""", (nodeid,))
@@ -466,13 +310,16 @@ class NoteBookIndex (object):
         else:
             return 0.0
 
-    def set_node_mtime(self, nodeid, mtime=None):
+    def set_node_mtime(self, nodeid, mtime=None, commit=True):
+        """Set the last indexed mtime for a node"""
 
         if mtime is None:
             mtime = time.time()
 
-        self.con.execute("""UPDATE NodeGraph SET mtime = ? WHERE nodeid = ?;""",
+        self.cur.execute("""UPDATE NodeGraph SET mtime = ? WHERE nodeid = ?;""",
                          (mtime, nodeid))
+        if commit:
+            self.con.commit()
 
 
     def get_mtime(self):
@@ -480,14 +327,14 @@ class NoteBookIndex (object):
         return os.stat(self._index_file).st_mtime
 
     
-    def add_node(self, nodeid, parentid, basename, attr, mtime):
+    def add_node(self, nodeid, parentid, basename, attr, mtime, commit=True):
         """Add a node to the index"""               
         
         # TODO: remove single parent assumption        
         
         if self.con is None:
             return
-
+        
         try:
             # get info
             if parentid is None:
@@ -500,21 +347,18 @@ class NoteBookIndex (object):
                 u"""INSERT INTO NodeGraph VALUES (?, ?, ?, ?, ?)""", 
                 (nodeid, parentid, basename, mtime, symlink))
 
-            # update attrs
-            for attrindex in self._attrs.itervalues():
-                attrindex.add_node(self.cur, nodeid, attr)
-
-            # update fulltext
-            infile = read_data_as_plain_text(self._nconn, nodeid)
-            self.index_node_text(nodeid, attr, infile)
+            self.add_node_attr(self.cur, nodeid, attr)
             
+            if commit:
+                self.con.commit()
+
         except Exception, e:
             keepnote.log_error("error index node %s '%s'" % 
                                (nodeid, attr.get("title", "")))
             self._on_corrupt(e, sys.exc_info()[2])
 
 
-    def remove_node(self, nodeid):
+    def remove_node(self, nodeid, commit=True):
         """Remove node from index using nodeid"""
 
         if self.con is None:
@@ -524,43 +368,15 @@ class NoteBookIndex (object):
             # delete node
             self.cur.execute(u"DELETE FROM NodeGraph WHERE nodeid=?", (nodeid,))
 
-            # update attrs
-            for attr in self._attrs.itervalues():
-                attr.remove_node(self.cur, nodeid)
+            self.remove_node_attr(self.cur, nodeid)
 
-            # delete children
-            #for (childid,) in self.cur.execute(
-            #    u"SELECT nodeid FROM NodeGraph WHERE parentid=?", (nodeid,)):
-            #    self.remove_node(childid)
+            if commit:
+                self.con.commit()
 
         except sqlite.DatabaseError, e:
             self._on_corrupt(e, sys.exc_info()[2])
 
 
-    def index_node_text(self, nodeid, attr, infile):
-
-        try:
-            text = attr.get("title", "") + "\n" + "".join(infile)
-            self.insert_text(nodeid, text)
-        except Exception, e:
-            keepnote.log_error()
-
-
-    def insert_text(self, nodeid, text):
-        
-        if not self._has_fulltext:
-            return
-
-        if list(self.cur.execute(u"SELECT 1 FROM fulltext WHERE nodeid = ?",
-                                 (nodeid,))):
-            self.cur.execute(
-                u"UPDATE fulltext SET content = ? WHERE nodeid = ?;",
-                (text, nodeid))
-        else:
-            self.cur.execute(u"INSERT INTO fulltext VALUES (?, ?);",
-                             (nodeid, text))
-
-
 
     #-------------------------
     # queries
@@ -672,6 +488,11 @@ class NoteBookIndex (object):
             self._on_corrupt(e, sys.exc_info()[2])
             raise
 
+
+    def get_attr(self, nodeid, attr):
+        """Return a nodes's attribute value"""
+        return self.get_node_attr(self.cur, nodeid, attr)
+
         
     def has_node(self, nodeid):
         """Returns True if index has node"""
@@ -682,7 +503,8 @@ class NoteBookIndex (object):
 
 
     def list_children(self, nodeid):
-                
+        """List children indexed for node"""
+
         try:
             self.cur.execute(u"""SELECT nodeid, basename
                                 FROM NodeGraph
@@ -695,6 +517,7 @@ class NoteBookIndex (object):
 
 
     def has_children(self, nodeid):
+        """Returns True if node has children"""
         
         try:
             self.cur.execute(u"""SELECT nodeid
@@ -707,83 +530,27 @@ class NoteBookIndex (object):
             raise
 
 
-    def search_titles(self, query):
-        """Return nodeids of nodes with matching titles"""
-
-        if "title" not in self._attrs:
-            return []
+    def search_titles(self, title):
+        """Search node titles"""
 
         try:
-            # order titles by exact matches and then alphabetically
-            self.cur.execute(
-                u"""SELECT nodeid, value FROM %s WHERE value LIKE ?
-                           ORDER BY value != ?, value """ % 
-                self._attrs["title"].get_table_name(),
-                (u"%" + query + u"%", query))
-
-            return list(self.cur.fetchall())
-
+            return self.search_node_titles(self.cur, title)
         except sqlite.DatabaseError, e:
             self._on_corrupt(e, sys.exc_info()[2])
             raise
 
 
-    def get_attr(self, nodeid, key):
-        """Query indexed attribute for a node"""
-        attr = self._attrs.get(key, None)
-        if attr:
-            return attr.get(self.cur, nodeid)
-        else:
-            return None
-
-
     def search_contents(self, text):
+        """Search node contents"""
 
-        # TODO: implement fully general fix
-        # crude cleaning
-        text = text.replace('"', "")
-
-        # fallback if fts3 is not available
-        if not self._has_fulltext:
-            words = [x.lower() for x in text.strip().split()]
-            return self._search_manual(words)
-        
         cur = self.con.cursor()
-
-        # search db with fts3
         try:
-            res = cur.execute("""SELECT nodeid FROM fulltext 
-                             WHERE content MATCH ?;""", (text,))
-            return (row[0] for row in res)
+            for res in self.search_node_contents(cur, text):
+                yield res            
         except:
             keepnote.log_error("SQLITE error while performing search")
-            return []
-
-
-    def _search_manual(self, words):
-        """Recursively search nodes under node for occurrence of words"""
-
-        keepnote.log_message("manual search")
-
-        nodeid = self._nconn.get_rootid()
-        
-        stack = [nodeid]
-        while len(stack) > 0:
-            nodeid = stack.pop()
-            
-            title = self._nconn.read_node(nodeid).get("title", "").lower()
-            infile = chain([title], 
-                           read_data_as_plain_text(self._nconn, nodeid))
-
-            if match_words(infile, words):
-                yield nodeid
-            else:
-                # return frequently so that search does not block long
-                yield None
-
-            children = self._nconn._list_children_nodeids(nodeid)
-            stack.extend(children)
-
+        finally:
+            cur.close()
 
 
 
